% Calculate draws of standardized random terms for random coefficients and save if specified
% Initially Written by Kenneth Train, August 6, 2006.
% Modified to be a parameterized function for generic use
% all global variables have been removed
% October 3, 2013
%
% Array of draws has dimension NMEMxNPxNV.


function dr=makedraws2(NDRAWS,NV,NP,DRAWTYPE,TRIAN)
 
% function dr=makedraws2(NDRAWS,NV,NP,DRAWTYPE,TRIAN)
%
% NDRAWS    : Number of draws from to use per person in simulation.
% NV        : Dimension (number of variables)
% NP        : Number of persons
% DRAWTYPE: 
        % Type of draws to use in simulation
        % 1=pseudo-random draws
        % 2=standard Halton draws
        % 3=shifted and shuffled Halton draws
        % 4=modified Latin hypercube sampling, shifted and shuffled 
 
% TRIAN     : Triangular  (replace IDV=6) 
% to get another order dr2=permute(dr,[1 3 2]);
%
%
if nargin<5
    TRIAN=0; % default, not triangular
end
  
if nargin<4
DRAWTYPE=3;
end


% Set seed for the random number generator.
SEED1 = 14239; 

% Memory use
% Give the number of draws that you want held in memory at one time.
% This number must be evenly divisible into the number of draws.
% That is NDRAWS./NMEM must be a positive integer.
% To hold all draws in memory at once, set NMEM=NDRAWS.
% A larger value of NMEM requires fewer reads from disc but 
% uses more memory which can slow-down the calculations and increases 
% the chance of running out of memory.
NMEM=NDRAWS;

% If all the draws are NOT held in memory at one time (that is, if NMEM<NDRAWS), 
% then give the filename (including full path if not in the working directory)
% that you want the draws to be temporarily saved to while the code is running.
% If all draws are held in memory at one time (that is, if NMEM=NDRAWS),
% then this file will not be created. So, if NMEM=NDRAWS, you can set PUTDR=''; 
% or give a file name, whichever you find more convenient, since the name won't be used.
PUTDR='draws';

NTAKES=NDRAWS./NMEM; %This tells code how many passes through the draws are needed 
                           % given NMEM people in each pass.



if NTAKES>1
  fid=fopen(PUTDR,'w'); 
  dr=[];
else
  dr=zeros(NMEM,NP,NV);
end


if DRAWTYPE == 1              %Random draws
  if NTAKES == 1              %Create all at once
      for j=1:NV
        if TRIAN ~=1              %Based on normal: not triangular
           dr(:,:,j)=randn(NMEM,NP);
        else                         %Triangular
           draws=rand(NMEM,NP);
           dr(:,:,j)=(sqrt(2.*draws)-1) .* (draws<=.5) + (1-sqrt(2.*(1-draws))) .* (draws >.5);
        end
     end
  else                       %Create and write NMEM draws for each person
                             %   since draws are to be read as NMEMxNP for each NV 
     for j=1:NV
         for p=1:NP
             for k=1:NTAKES
                if TRIAN~=1
                   draws=randn(NMEM,1);
                else
                   draws=rand(NMEM,1);
                   draws=(sqrt(2.*draws)-1) .* (draws<=.5) + (1-sqrt(2.*(1-draws))) .* (draws >.5);
                end
                fwrite(fid,draws,'double');
             end
         end
     end
   end
end

if DRAWTYPE == 2 | DRAWTYPE == 3   % Halton draws
   h=primes(100);                                % Must create for all people together
   k=1;
   while size(h,2) < NV
       h=primes(k.*100);
       k=k+1;
   end
   h=h(1,1:NV);
   for j=1:NV
       hh=h(1,j);
       draws=[0];
       test=0;
       b=1;
       while test == 0
            drawsold=draws;
            for m=1:(hh-1);
                dd=m./(hh.^b);
                draws=[draws ; drawsold + dd];
                test=size(draws,1) >= ((NP.*NDRAWS) + 10);
                if test == 1
                   break
                end
            end
            b=b+1;    
       end
       draws=draws(11:(NP.*NDRAWS)+10,1);
       if DRAWTYPE == 3
            draws=draws+rand(1,1);               %Shift: one shift for entire sequence
            draws=draws-floor(draws);
            draws=reshape(draws,NDRAWS,NP);
            for n=1:NP                           %Shuffle for each person separately
               rr=rand(NDRAWS,1);
               [rr rrid]=sort(rr);
               draws(:,n)=draws(rrid,n);
            end;
            draws=reshape(draws,NP.*NDRAWS,1);
       end
       if TRIAN~=1
           draws=-sqrt(2).*erfcinv(2.*draws);  %Take inverse cum normal
       else
           draws=(sqrt(2.*draws)-1) .* (draws<=.5) + (1-sqrt(2.*(1-draws))) .* (draws >.5); 
       end
       if NTAKES == 1
           dr(:,:,j)=reshape(draws,NDRAWS,NP);
           
       else
           fwrite(fid,draws,'double');
       end
    end
end

if DRAWTYPE == 4   % MLHS

  h=0:(NDRAWS-1);
  h=h'./NDRAWS;
  for j=1:NV
    for n=1:NP
       draws=h+rand(1,1)./NDRAWS;    %Shift: Different shift for each person
       rr=rand(NDRAWS,1);
       [rr rrid]=sort(rr);
       draws=draws(rrid,1);          %Shuffle
       if TRIAN~=1
           draws=-sqrt(2).*erfcinv(2.*draws);  %Take inverse cum normal
       else
           draws=(sqrt(2.*draws)-1) .* (draws<=.5) + (1-sqrt(2.*(1-draws))) .* (draws >.5); 
       end
       if NTAKES == 1
         dr(:,n,j)=draws;
       else
         fwrite(fid,draws,'double');
       end
     end
   end
end




if NTAKES>1
  fclose(fid);
end


